/* 
   $Id: nnmdicman.c,v 1.3 2002/01/31 11:09:15 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome.

   Nunome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xlocale.h>

#include <X11/Composite.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/AsciiText.h>

#include "Nnm.h"
#include "NnmProto.h"
#include "nnmCanvas.h"
#include "nnmUtil.h"
#include "klist.h"

static long	editcode = 0x0;
static int nnmsrvPID = 0;

char		*dicfn=NULL;			/* ե̾ */


static	void	codeChanged( w, clo, cli )
     Widget	w;
     XtPointer	clo, cli;
{
    String p;

    XtVaGetValues(w,XtNstring,&p,NULL);
    if ((p[0] & 0xff) >= 0x80) {
	editcode = ((p[0] & 0x7f) << 8)|(p[1] & 0x7f);
    } else {
	editcode = (p[0] & 0xff);
    }
}

static	void	NnmAdd( w, clo, cli )
     Widget	w;
     XtPointer	clo, cli;
{
    NnmFeatureRec fv;
    NnmCharInfoRec ci;
    NnmFeatureDicRec dicfv;
    NnmCanvasRec *p = (NnmCanvasRec *)clo;
    int fd;
    int found = 0;
    long fpos = 0;

    if (p->clean || editcode == 0) return;
    NnmConvertCharInfoToFeature(&(p->CharInfo),&ci,&fv);
    fd = NnmFOpen(dicfn,"r+");
    while (!NnmFEof(fd)) {
	fpos = NnmFTell(fd);
	NnmReadStandardFeature(fd,&dicfv);
	if (dicfv.code == editcode) {
	    found = 1;
	    break;
	}
    }
    if (found) {
	NnmUpdateFeatureDic(&fv,&dicfv);
	NnmFSeek(fd,fpos,SEEK_SET);
    } else {
	NnmFeatureToFeatureDic(editcode,&fv,&dicfv);
	NnmFSeek(fd,0,SEEK_END);
    }
    NnmWriteStandardFeature(fd,&dicfv);
    NnmFClose(fd);
    NnmCanvasClear(p);
}

/*
 * ʸȤΥꥢ
 * Clear Button Callback
 */
static	void	NnmClear( w, clo, cli )
     Widget	w;
     XtPointer	clo, cli;
{
    NnmCanvasClear((NnmCanvasRec *)clo);
}

/* Quit Button Callback */
static	void	Quit()
{
  if (nnmsrvPID != 0) {
    kill(nnmsrvPID,SIGHUP);
  }
  exit(0);
}

static void usage(char *cmd)
{
    fprintf(stderr,"Usage:%s [-c <code>][-p <nnmsrv PID>] <dic filename>\n",cmd);
    exit(2);
}

static void parseOpt(int argc, char **argv)
{
    int i;

    for (i = 1;i < argc;i++) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
	    case 'p':
		if (++i < argc) {
		    nnmsrvPID = atoi(argv[i]);
		} else {
		    usage(argv[0]);
		}
		break;
	    case 'c':
		if (++i < argc) {
		    editcode = strtol(argv[i],NULL,0);
		} else {
		    usage(argv[0]);
		}
		break;
	    case 'h':
	    default:
		usage(argv[0]);
		break;
	    }
	} else {
	    dicfn = argv[i];
	}
    }
    if (dicfn == NULL) {
	usage(argv[0]);
    }
}

Widget	toplevel, topBox, klistBox;
Widget	codeInput;

void NnmGetShellPos(int *x, int *y, int *w, int *h)
{
    NnmGetWindowPos(toplevel, x, y, w, h);
}

void switchToTop(void)
{
    XtUnmapWidget(klistBox);
    XtMapWidget(topBox);
}

void switchToKlist(void)
{
  XtUnmapWidget(topBox);
  XtMapWidget(klistBox);
}

void setCode(char *p)
{
    XtVaSetValues(codeInput,XtNstring,p,NULL);
    codeChanged(codeInput,NULL,NULL);
}

static	void	NnmKlist( w, clo, cli )
     Widget	w;
     XtPointer	clo, cli;
{
    switchToKlist();
}

int	main( argc, argv )
     int	argc;
     char	**argv;
{
  XtAppContext	app_con;
  Widget	AddBtn, ClearBtn, QuitBtn, KlistBtn;
  Widget	dispBox,inputBox,buttonBox;
  NnmCanvasRec	canvasInfo;
  Arg		args[10];
  Cardinal	n;
  char		buf[3];
    
  /* International */
  setlocale( LC_ALL, "" );
  XtSetLanguageProc( NULL, NULL, NULL );
  toplevel = XtAppInitialize(&app_con, "Nnmdicman", NULL, 0,
			     &argc, argv, NULL, NULL, (Cardinal) 0);

  parseOpt(argc,argv);
  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  topBox = XtCreateManagedWidget( "topBox", boxWidgetClass, toplevel,
			       args, n );
  /* ʸɽWidget */
  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  XtSetArg( args[n], XtNorientation , XtorientHorizontal ); n++;
  dispBox = XtCreateManagedWidget( "dispBox", boxWidgetClass, topBox,
			       args, n );
  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  XtCreateManagedWidget( "char", labelWidgetClass, dispBox,
				      args, n );
  n = 0;
  XtSetArg( args[n], XtNheight,   21 ); n++;
  XtSetArg( args[n], XtNwidth ,   40 ); n++;
  XtSetArg( args[n], XtNeditType ,   XawtextEdit ); n++;
  XtSetArg( args[n], XtNwrap ,   XawtextWrapNever ); n++;
  if (editcode != 0) {
    if ((editcode & 0xff00) == 0) {
	sprintf(buf,"%c",(int)(editcode & 0xff));
    } else {
	sprintf(buf,"%c%c",(int)((editcode & 0xff00) >> 8) | 0x80,
	  (int)(editcode & 0xff) | 0x80);
    }
    XtSetArg( args[n], XtNstring ,  buf ); n++;
  }
  codeInput = XtCreateManagedWidget( "codeInput", asciiTextWidgetClass, dispBox,
				      args, n );
  /* ʸWidget */
  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  XtSetArg( args[n], XtNorientation , XtorientHorizontal ); n++;
  inputBox = XtCreateManagedWidget( "inputBox", boxWidgetClass, topBox,
			       args, n );
  n = 0;
  XtSetArg( args[n], XtNwidth , NnmCANVAS_DISPLAY_SIZE ); n++;
  XtSetArg( args[n], XtNheight, NnmCANVAS_DISPLAY_SIZE ); n++;
  NnmCreateCanvas( &canvasInfo, 0, inputBox, args, n );

  /* Button Widgets */
  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  buttonBox = XtCreateManagedWidget( "buttonBox", boxWidgetClass, inputBox,
			       args, n );
  n = 0;
  KlistBtn = XtCreateManagedWidget( "klist", commandWidgetClass, buttonBox,
				    args, n );
  n = 0;
  AddBtn = XtCreateManagedWidget( "Add", commandWidgetClass, buttonBox,
				    args, n );
  /* ʸΥȥξõܥ(ľ) */
  n = 0;
  ClearBtn = XtCreateManagedWidget( "Clear", commandWidgetClass, buttonBox,
				    args, n );
  /* ץꥱλܥ */
  n = 0;
  QuitBtn = XtCreateManagedWidget( "Quit", commandWidgetClass, buttonBox,
				    args, n );
  /* Button WidgetsCallbackؿϿ */
  XtAddCallback( AddBtn, XtNcallback, NnmAdd, &canvasInfo );
  XtAddCallback( ClearBtn, XtNcallback, NnmClear, &canvasInfo );
  XtAddCallback( KlistBtn, XtNcallback, NnmKlist, &canvasInfo );
  XtAddCallback( QuitBtn, XtNcallback, Quit, &canvasInfo );
  {
      Widget src;

      src = XawTextGetSource(codeInput);
      XtAddCallback(src, XtNcallback, codeChanged, NULL);
  }

  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  klistBox = XtCreateManagedWidget( "klistBox", boxWidgetClass, toplevel,
			       args, n );
  n = 0;
  XtSetArg( args[n], XtNborderWidth , 0 ); n++;
  CreateKigouWidgets( "kigouform",
		      klistBox, args, n );
  XtSetKeyboardFocus(topBox,codeInput);

  XtRealizeWidget(toplevel);
  XtAppMainLoop(app_con);
  return 0;
}
