/* 
   $Id: nnmincmkdic.c,v 1.2 2001/07/30 07:21:22 sho Exp $

   Copyright (C) 2001 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome.

   Nunome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <ctype.h>
#include "Nnm.h"
#include "NnmProto.h"
#include "NnmFeature.h"

static char	*dicfn, *listfn;
static int onlyone = 0;

static	int	NnmCreateStandardFeature( fn, stdfv )
     char		*fn;
     NnmFeatureDicRec	*stdfv;
{
  int			d, x, y, mi, mj;
  int			infd;
  NnmRawCharInfoRec	charInfo;
  NnmFeatureRec		fv;
  NnmCharInfoRec	ci;
  ulong			num_sample, width, height, num_stroke;
#ifdef CAND_TEST
  ulong			strkLong[8][NnmCAND_SPLIT_NUM];
#ifdef CAND_TEST2
  ulong			strkShort[8][NnmCAND_SPLIT_NUM][NnmCAND_SPLIT_NUM];
#else
  ulong			strkShort[8][NnmCAND_SPLIT_NUM];
#endif // CAND_TEST2
#endif // CAND_TEST
  ulong			dir_fv[8][NnmFV_REGION_Y][NnmFV_REGION_X];
  ulong			dir_stroke[8];
  ulong			code;
  
  /* Initialize */
  num_sample = height = width = num_stroke = 0;
#ifdef CAND_TEST
  memset( strkLong, 0, sizeof(strkLong) );
  memset( strkShort, 0, sizeof(strkShort) );
#endif
  for( d=0; d<8; d++ ){
    dir_stroke[d] = 0;
#ifdef NNM_WEIGHT
    stdfv->Cand.dir_stroke_weight[d] = NNM_WEIGHT_STD;
#endif
    for( y=0; y<NnmFV_REGION_Y; y++ ){
      for( x=0; x<NnmFV_REGION_X; x++ ){
	dir_fv[d][y][x] = 0;
#ifdef NNM_WEIGHT
	stdfv->weight[d][y][x] = NNM_WEIGHT_STD;
#endif
      }
    }
  }
  /* Stroke Load */
  if ( (infd=NnmFOpen( fn, "r" ))==-1 ){
    fprintf(stderr,"File Error: Cannot open sample vector file[%s]\n",fn);
    return -1;
  }
  for( num_sample=0; !NnmFEof(infd) ; ){
    if ( NnmSampleReadFile( &charInfo, &code, infd )==-1 ) break;
    NnmConvertCharInfoToFeature( &charInfo, &ci, &fv );
    num_sample++;
    width      += fv.Cand.sizeX;
    height     += fv.Cand.sizeY;
    num_stroke += fv.Cand.num_stroke;
    for( d=0; d<8; d++ ){
      dir_stroke[d] += fv.Cand.dir_stroke[d];
#ifdef CAND_TEST
      for( mi=0; mi<NnmCAND_SPLIT_NUM; mi++ ){
	strkLong[d][mi] += fv.Cand.strkLong[d][mi];
#ifdef CAND_TEST2
	for( mj=0; mj<NnmCAND_SPLIT_NUM; mj++ ){
	  strkShort[d][mi][mj] += fv.Cand.strkShort[d][mi][mj];
	}
#else
	strkShort[d][mi] += fv.Cand.strkShort[d][mi];
#endif	// CAND_TEST2
      }
#endif	// CAND_TEST
      for( y=0; y<NnmFV_REGION_Y; y++ ){
	for( x=0; x<NnmFV_REGION_X; x++ ){
	  dir_fv[d][y][x] += fv.dir_fv[d][y][x];
	}
      }
    }
    if (onlyone) break;
  }
  NnmFClose( infd );
  /*
  printf("File[%s]: num(%4d)  strk(%4d)  (%4d x %4d)\n",
	 fn, num_sample, num_stroke, width, height );
  */  
  if ( num_sample<=0 ) return 0;

  /* Standard Feature */
  stdfv->num_sample = num_sample;
  stdfv->code       = code;
  stdfv->Cand.sizeX = width / num_sample;
  stdfv->Cand.sizeY = height / num_sample;
  stdfv->Cand.num_stroke = num_stroke / num_sample;
  for( d=0; d<8; d++ ){
    stdfv->Cand.dir_stroke[d] = dir_stroke[d] / stdfv->num_sample;
#ifdef CAND_TEST
      for( mi=0; mi<NnmCAND_SPLIT_NUM; mi++ ){
	stdfv->Cand.strkLong[d][mi] = strkLong[d][mi] / num_sample;
#ifdef CAND_TEST2
	for( mj=0; mj<NnmCAND_SPLIT_NUM; mj++ ){
	  stdfv->Cand.strkShort[d][mi][mj] = strkShort[d][mi][mj] / num_sample;
	}
#else
	stdfv->Cand.strkShort[d][mi] = strkShort[d][mi] / num_sample;
#endif	// CAND_TEST2
      }
#endif	// CAND_TEST
    for( y=0; y<NnmFV_REGION_Y; y++ ){
      for( x=0; x<NnmFV_REGION_X; x++ ){
	stdfv->dir_fv[d][y][x] = dir_fv[d][y][x] / stdfv->num_sample;
      }
    }
  }
  return 0;
}

static	void	NnmAppendDictionary( fn, stdfv )
     char		*fn;
     NnmFeatureDicRec	*stdfv;
{
  int			dicfd = -1;
  NnmFeatureDicRec	dicfv, resfv;
  int			n, d, x, y, idx, mi, mj;
  long			fpos;
  long			tmp;
  long			half;
  
  if ( (dicfd=NnmFOpen( fn, "r+"))==-1 ){	/* file is not exist, and create it */
    dicfd = NnmFOpen( fn, "w" );
    NnmWriteStandardFeature( dicfd, stdfv );
    NnmFClose( dicfd );
    return ;
  }else{
    /* category search */
    for( idx=0; !NnmFEof(dicfd); idx++ ){
      fpos = NnmFTell( dicfd );
      n = NnmReadStandardFeature( dicfd, &dicfv );
      if ( dicfv.code==stdfv->code ){
	NnmFSeek( dicfd, fpos, SEEK_SET );
	/* merge */
	resfv.code       = stdfv->code;
	resfv.num_sample = dicfv.num_sample + stdfv->num_sample;
	half = resfv.num_sample/2;
	tmp = (int)dicfv.Cand.sizeX*dicfv.num_sample +
			    (int)stdfv->Cand.sizeX*stdfv->num_sample+half;
	resfv.Cand.sizeX = tmp/resfv.num_sample;
	tmp = (int)dicfv.Cand.sizeY*dicfv.num_sample +
			    (int)stdfv->Cand.sizeY*stdfv->num_sample+half;
	resfv.Cand.sizeY = tmp/resfv.num_sample;
	tmp = (int)dicfv.Cand.num_stroke*dicfv.num_sample +
	   (int)stdfv->Cand.num_stroke*stdfv->num_sample+half;
	resfv.Cand.num_stroke = tmp/resfv.num_sample;
#ifdef CAND_TEST
	for( d=0; d<8; d++ ){
	  for( mi=0; mi<NnmCAND_SPLIT_NUM; mi++ ){
	    tmp = (int)dicfv.Cand.strkLong[d][mi]*dicfv.num_sample +
	       (int)stdfv->Cand.strkLong[d][mi]*stdfv->num_sample+half;
	    resfv.Cand.strkLong[d][mi] = tmp/resfv.num_sample;
#ifdef CAND_TEST2
	    for( mj=0; mj<NnmCAND_SPLIT_NUM; mj++ ){
	      tmp = (int)dicfv.Cand.strkShort[d][mi][mj]*dicfv.num_sample +
		 (int)stdfv->Cand.strkShort[d][mi][mj]*stdfv->num_sample+half;
	      resfv.Cand.strkShort[d][mi][mj] = tmp/resfv.num_sample;
	    }
#else
	    tmp = (int)dicfv.Cand.strkShort[d][mi]*dicfv.num_sample +
	       (int)stdfv->Cand.strkShort[d][mi]*stdfv->num_sample+half;
	    resfv.Cand.strkShort[d][mi] = tmp/resfv.num_sample;
#endif	// CAND_TEST2
	  }
	}
#endif	// CAND_TEST
	for( d=0; d<8; d++ ){
	  tmp = (int)dicfv.Cand.dir_stroke[d]*dicfv.num_sample +
	     (int)stdfv->Cand.dir_stroke[d]*stdfv->num_sample+half;
	  resfv.Cand.dir_stroke[d] = tmp/resfv.num_sample;
#ifdef NNM_WEIGHT
	  resfv.Cand.dir_stroke_weight[d] = NNM_WEIGHT_STD;
#endif
	  for( y=0; y<NnmFV_REGION_Y; y++ ){
	    for( x=0; x<NnmFV_REGION_X; x++ ){
	      tmp = (int)dicfv.dir_fv[d][y][x]*dicfv.num_sample +
		       (int)stdfv->dir_fv[d][y][x]*stdfv->num_sample+half;
	      resfv.dir_fv[d][y][x] = tmp/resfv.num_sample;
#ifdef NNM_WEIGHT
	      resfv.weight[d][y][x] = NNM_WEIGHT_STD;
#endif
	    }
	  }
	}
	NnmWriteStandardFeature( dicfd, &resfv );
	NnmFClose( dicfd );
	return;
      }
    }
    /* replace */
    NnmWriteStandardFeature( dicfd, stdfv );
  }
  NnmFClose( dicfd );
  return;
}

static void usage(char *cmd)
{
    fprintf(stderr,"Usage:%s [-1][<dic filename>][<sample list file name>]\n",cmd);
    exit(2);
}

static void parseOpt(int argc, char **argv)
{
    int i;
    int n = 0;

    for (i = 1;i < argc;i++) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
	    case '1':
		onlyone++;
		break;
	    default:
		usage(argv[0]);
		break;
	    }
	} else {
	    switch (n++) {
	    case 0:
		dicfn = argv[i];
		break;
	    case 1:
		listfn = argv[i];
		break;
	    default:
		usage(argv[0]);
		break;
	    }
	}
    }
    if (n < 2) {
	usage(argv[0]);
    }
}

int	main( argc, argv )
     int	argc;
     char	**argv;
{
  int			n;
  FILE			*listfp;
  char			sampfn[MAX_STR_LEN], buf[MAX_STR_LEN];
  NnmFeatureDicRec	stdfv;
  
  /* Usage */
  parseOpt(argc,argv);
  if ( (listfp=fopen( listfn, "r" ))==NULL ){
    fprintf( stderr, "Sample List File is not exist [%s]\n",
	     listfn );
    exit(1);
  }
  for( n=0; !feof(listfp); n++ ){
    if ( fgets( buf, MAX_STR_LEN-1, listfp )==NULL ){
      break;
    }
    sscanf( buf, "%s", sampfn );
    printf("%s    \r",sampfn);fflush(stdout);
    NnmCreateStandardFeature( sampfn, &stdfv );
    NnmAppendDictionary( dicfn, &stdfv );
  }
  return 0;
}
