/* 
   $Id: nnmcheck.c,v 1.3 2001/07/30 07:21:22 sho Exp $

   Copyright (C) 2001 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome.

   Nunome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xlocale.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>

#include "Nnm.h"
#include "NnmProto.h"
#include "nnmCanvas.h"
#include "nnmUtil.h"

#define MAX_STR_LEN		2048

/* ʸ󼨤뤿Window Size */
#define NnmDISP_KANJI_SIZE	20

Widget		dispWidget;	/* ʸ,ǧɽWidget*/
XFontStruct	*kfs = NULL;			/* ɽFont */

Widget		fnLabel;

char		sampfn[2048];
int		sampfd = -1;
FILE		*listfp;
unsigned long	c_code;
NnmCanvasRec	canvasInfo1;

static	int	LoadNext( NnmCanvasRec *canvasInfo );
static int	NextSampleFile();


static void	SetLabel( char *fn )
{
  Arg	args[10];

  XtSetArg( args[0], XtNlabel, fn );
  XtSetValues( fnLabel, args, 1 );
}


static	int	LoadNext( NnmCanvasRec *canvasInfo )
{
  if ( NnmSampleReadFile(&(canvasInfo->CharInfo), &c_code, sampfd )==-1 ){
    NextSampleFile();
  }
  canvasInfo->clean = False;
  return 0;
}


static int	NextSampleFile()
{
  char	tmp[2048];

  fgets( tmp, 2047, listfp );
  sscanf( tmp, "%s", sampfn );
  if ( sampfd != -1 ) NnmFClose( sampfd );

  if ( (sampfd = NnmFOpen( sampfn, "r" ))==-1 ){
    fprintf( stderr, "Error: Cannot open Sample File [%s]\n",sampfn );
    return -1;
  }
  SetLabel( sampfn );
  LoadNext( &canvasInfo1 );
  return 0;
}


static void	InitFile ( char *fn )
{
  if ( (listfp=fopen( fn, "r" ))==NULL ){
    fprintf( stderr, "Error: Cannot open List File [%s]\n",fn );
  }
  if ( NextSampleFile() ) exit(1);
}



/* dispWidgetκ襤٥ȥХå */
static	void	dispExposure( w, clo, ev, ctd )
     Widget	w;
     XtPointer	clo;
     XEvent	*ev;
     Boolean	*ctd;
{
  NnmDisplayKanjiChar( dispWidget, c_code, kfs, 1, NnmDISP_KANJI_SIZE-5 );
}

static void goNextChar()
{
  NnmCanvasClear(&canvasInfo1);
  LoadNext( &canvasInfo1 );
  NnmDisplayKanjiChar( dispWidget, c_code, kfs, 1, NnmDISP_KANJI_SIZE-5 );
  NnmCanvasDraw(&canvasInfo1);
}


static	void	OKBtnProc( Widget w, XtPointer clo, XtPointer cli )
{
  char	outfn[2048];

  sprintf( outfn, "%s.OK", sampfn );
  NnmSampleAppendFile( &canvasInfo1.CharInfo, c_code, outfn );  
  goNextChar();
}


static	void	KeepBtnProc( Widget w, XtPointer clo, XtPointer cli )
{
  char	outfn[2048];

  sprintf( outfn, "%s.Keep", sampfn );
  NnmSampleAppendFile( &canvasInfo1.CharInfo, c_code, outfn );  
  goNextChar();
}


static	void	NGBtnProc( Widget w, XtPointer clo, XtPointer cli )
{
  char	outfn[2048];

  sprintf( outfn, "%s.NG", sampfn );
  NnmSampleAppendFile( &canvasInfo1.CharInfo, c_code, outfn );  
  goNextChar();
}

static	void	SkipBtnProc( Widget w, XtPointer clo, XtPointer cli )
{
  goNextChar();
}


static	void	QuitBtnProc( Widget w, XtPointer clo, XtPointer cli )
{
  exit( 0 );
}

static void usage(char *cmd)
{
    fprintf(stderr,"Usage:%s <sample file list file>\n",cmd);
    exit(2);
}

int	main( argc, argv )
     int	argc;
     char	**argv;
{
  XtAppContext	app_con;
  Widget	toplevel;
  Widget	topBox, dispBox, buttonBox, inputBox;
  Widget	OKBtn, KeepBtn, NGBtn, SkipBtn, QuitBtn;
  Arg		args[10];
  Cardinal	n;
    
  setlocale( LC_ALL, "" );
  XtSetLanguageProc( NULL, NULL, NULL );
  toplevel = XtAppInitialize(&app_con, "Nnmcheck", NULL, 0,
			     &argc, argv, NULL, NULL, (Cardinal) 0);

  if (argc < 2) {
    usage(argv[0]);
  }
  n = 0;
  XtSetArg( args[n], XtNborderWidth,   0 ); n++;
  topBox = XtCreateManagedWidget( "topBox", boxWidgetClass, toplevel,
			       args, n );

  /* ʸɽWidget */
  n = 0;
  XtSetArg( args[n], XtNborderWidth,   0 ); n++;
  dispBox = XtCreateManagedWidget( "dispBox", boxWidgetClass, topBox,
			       args, n );
  n = 0;
  XtSetArg( args[n], XtNlabel, "FileName:" );     	n++;
  fnLabel = XtCreateManagedWidget( "fnlabel", labelWidgetClass, dispBox,
				   args, n );
  n = 0;
  XtSetArg( args[n], XtNheight,   NnmDISP_KANJI_SIZE ); n++;
  XtSetArg( args[n], XtNwidth ,   NnmDISP_KANJI_SIZE ); n++;
  dispWidget = XtCreateManagedWidget( "disp", simpleWidgetClass, dispBox,
				      args, n );
  XtAddEventHandler( dispWidget, ExposureMask, (Boolean)0, 
		     (XtEventHandler)dispExposure, (XtPointer)NULL );
  /* Create Canvas Window */
  n = 0;
  XtSetArg( args[n], XtNborderWidth,   0 ); n++;
  inputBox = XtCreateManagedWidget( "inputBox", boxWidgetClass, topBox,
			       args, n );
  n = 0;
  XtSetArg( args[n], XtNwidth , NnmCANVAS_DISPLAY_SIZE ); n++;
  XtSetArg( args[n], XtNheight, NnmCANVAS_DISPLAY_SIZE ); n++;
  XtSetArg( args[n], XtNsensitive, False ); n++;
  NnmCreateCanvas( &canvasInfo1, 0,inputBox, args, n );

  /* Button windows */
  buttonBox = XtCreateManagedWidget( "buttonBox", boxWidgetClass, topBox,
			       NULL, 0 );
  n = 0;
  XtSetArg( args[n], XtNlabel, "OK" );     	n++;
  OKBtn = XtCreateManagedWidget( "okbtn", commandWidgetClass, buttonBox,
				   args, n );

  n = 0;
  XtSetArg( args[n], XtNlabel, "Keep" );     n++;
  KeepBtn = XtCreateManagedWidget( "keepbtn", commandWidgetClass, buttonBox,
				   args, n );

  n = 0;
  XtSetArg( args[n], XtNlabel, "NG" );     n++;
  NGBtn = XtCreateManagedWidget( "ngbtn", commandWidgetClass, buttonBox,
				   args, n );

  n = 0;
  XtSetArg( args[n], XtNlabel, "Skip" );     n++;
  SkipBtn = XtCreateManagedWidget( "skipbtn", commandWidgetClass, buttonBox,
				   args, n );

  n = 0;
  XtSetArg( args[n], XtNlabel, "Quit" );     n++;
  QuitBtn = XtCreateManagedWidget( "quitbtn", commandWidgetClass, buttonBox,
				   args, n );
  /*  */
  XtAddCallback( OKBtn,   XtNcallback, OKBtnProc,   NULL );
  XtAddCallback( KeepBtn, XtNcallback, KeepBtnProc, NULL );
  XtAddCallback( NGBtn,   XtNcallback, NGBtnProc,   NULL );
  XtAddCallback( SkipBtn, XtNcallback, SkipBtnProc,   NULL );
  XtAddCallback( QuitBtn, XtNcallback, QuitBtnProc, NULL );
  XtRealizeWidget(toplevel);
  InitFile( argv[1] );
  NnmCanvasDraw(&canvasInfo1);
  XtAppMainLoop(app_con);
  return 0;
}
