/* 
   $Id: nnmsamls.c,v 1.1.1.1 2001/06/27 08:14:42 sho Exp $

   Copyright (C) 2001 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome.

   Nunome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>

#include "Nnm.h"
#include "NnmProto.h"

static char *filename = NULL;
static int longform = 0;
static int totalonly = 0;

static void usage(char *cmd)
{
    fprintf(stderr,"Usage:%s [-l][<sample file name>]\n",cmd);
    exit(2);
}

static void parseOpt(int argc, char **argv)
{
    int i;

    for (i = 1;i < argc;i++) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
	    case 'l':
		longform = 1;
		break;
	    case 't':
		totalonly = 1;
		break;
	    case 'h':
	    default:
		usage(argv[0]);
		break;
	    }
	} else {
	    filename = argv[i];
	}
    }
}

int main(int argc, char **argv)
{
    int fd = 0;
    NnmRawCharInfoRec charInfo;
    ulong code;
    int n;

    parseOpt(argc,argv);
    if (filename != NULL) {
	if ((fd = NnmFOpen(filename,"r")) < 0) {
	    fprintf(stderr,"Can't open sample file:%s\n",filename);
	    exit(2);
	}
    }
    if (totalonly) {
	n = 0;
	while (NnmSampleReadFile(&charInfo,&code,(ulong)fd) >= 0) {
	    n++;
	}
	printf("Number of characters is %d\n",n);
    } else if (longform) {
	int i;
	int ncoord;
	int maxnstr = 0;
	int maxnpt = 0;

	n = 0;
        printf("code char   width  height strokes  coords\n");
	while (NnmSampleReadFile(&charInfo,&code,(ulong)fd) >= 0) {
	    printf("%04lx ",code);
	    if ((code & 0xff00) != 0) {
		putchar((((code &0xff00) >> 8) & 0xff) | 0x80);
	    }
	    putchar((code & 0xff) | 0x80);
	    printf("  %8d%8d%8d",charInfo.width,
	      charInfo.height,charInfo.num);
	    if (maxnstr < charInfo.num) {
		maxnstr = charInfo.num;
	    }
	    ncoord = 0;
	    for (i = 0;i < charInfo.num;i++) {
		ncoord += charInfo.stroke[i].len;
		if (maxnpt < charInfo.stroke[i].len) {
		    maxnpt = charInfo.stroke[i].len;
		}
	    }
	    printf("%8d\n",ncoord);
	    n++;
	}
	printf("Number of characters is %d\n",n);
	printf("Max number of strokes is %d\n",maxnstr);
	printf("Max number of points is %d\n",maxnpt);
    } else {
	int t = 0;

	n = 0;
	while (NnmSampleReadFile(&charInfo,&code,(ulong)fd) >= 0) {
	    if (n >= 10) {
		printf("\n");
		n = 0;
	    }
	    if (n != 0) {
		printf(" ");
	    }
	    printf("%04lx",code);
	    n++;
	    t++;
	}
	printf("\n");
	printf("Number of characters is %d\n",t);
    }
    if (!NnmFEof(fd)) {
	fprintf(stderr,"Incomplete sample file\n");
	exit(2);
    }
    return 0;
}
